extproc perl -S
#!/usr/bin/perl

# Author: Xavier Noria <`echo -n ska@unfuers.pbz | tr a-z n-za-m`>
# Time-stamp: <2003-09-23 23:36:21 fxn>
# Modified (added some additional search expressions) <2003-09-24 fbakan@gmx.net>
use strict;
use warnings;

use Getopt::Std;
use Net::POP3;
use Term::ANSIColor qw(:constants);

sub say { print @_, "\n" }

# Process parameters.
my %params = ();
getopts('ns:l:p:', \%params);
my ($server, $login, $password) = @params{qw(s l p)};

defined $server and defined $login and defined $password or print <<USAGE and exit;
Usage: $0 -s server -l login -p password [-n]
USAGE

my $nonstop = exists $params{n};

# ----------------------------------------------------------------------
#
# This data is based on the worm exam published in
#
#      http://www.f-secure.com/v-descs/swen.shtml
#

my $swen_size = 106496; # bytes

my $from_markers = join "|", qw(
MS
Microsoft
Corporation
Program
Internet
Network
Security
Division
Section
Department
Center
Technical
Public
Customer
Bulletin
Services
Assistance
Support
mail
service
webdaemon
Administrator
system
System
Postmaster
Admin
netmail
System
rocketmail
@\S*(?:microsoft|msd?n?)\.(?:net|com)
);
my $from_regexp = qr/\b(?:$from_markers)\b/i;

my $to_markers = join "|", qw(
Commercial
MS
Microsoft
Corporation
Customer
User
Partner
Consumer
Client
receiver
client
user
recipient
inet
);
my $to_regexp = qr/\b(?:$to_markers)\b/i;

my $subject_markers = join "|", qw(
Current
Newest
Last
New
Latest
Net
Network
Microsoft
Internet
Critical
Security
Patch
Update
Pack
Upgrade
Message
Announcement
Failure
Notice
Report
Mail
);
my $subject_regexp = qr/\b(?:$subject_markers)\b/i;
#
# ----------------------------------------------------------------------


# Connect to the server.
my $pop;
eval { $pop = Net::POP3->new($server) };
defined $pop or print <<MSG and exit;
Couldn't connect to the POP3 server $server.
MSG

# Log in. Do not print login data to the screen if not possible.
my $nmails;
eval { $nmails = $pop->login($login, $password) };
defined $nmails or print <<MSG and exit;
I could connect to the server but couldn't log in.
MSG

# In the trials I've done $nmails is "0E0" (which is true in Perl) if
# there are no new mails, or a number otherwise. That's why an explicit
# numerical test is needed.
if ($nmails > 0) {
    my @deleted = ();
    my %messages = %{$pop->list};
    while (my ($msgnum, $size) = each %messages) {
        my @lines = @{$pop->top($msgnum)};
        # extract From and Subject headers
        my ($from)    = grep /^From:/i, @lines;
        my ($to)      = grep /^To:/i, @lines;
        my ($subject) = grep /^Subject:/i, @lines;
        # Any suspicious mail is at least as big as the worm itself.
        if ($size > $swen_size) {
            my $matches = 0;
            ++$matches if $from    =~ $from_regexp;
            ++$matches if $to      =~ $to_regexp;
            ++$matches if $subject =~ $subject_regexp;
            if ($matches >= 2) { # sometimes "To:" comes empty
                # highlight match if any, unless this is Windows
                unless ($^O =~ /win/i) {
                    $from    =~ s/$from_regexp/BOLD . RED . $& . RESET/geo;
                    $to      =~ s/$to_regexp/BOLD . RED . $& . RESET/geo;
                    $subject =~ s/$subject_regexp/BOLD . RED . $& . RESET/geo;
                }
                print "\nSize: @{[int($size/1024)]}K\n";
                print $from, $to, $subject;
                if ($nonstop) {
                    push @deleted, $msgnum;
                } else {
                    my $a;
                    do {
                        print 'Mark message for deletion? ([Y]es, [n]o, [a]bort): ';
                        $a = <STDIN>
                    } until $a =~ /^[yna]?$/i;
                    if ($a =~ /^y?$/i) {
                        push @deleted, $msgnum;
                    } elsif ($a =~ /^a$/i) {
                        say "Aborted. No messages were deleted from the server.";
                        $pop->quit;
                        exit;
                    }
                }
            }
        }
    }
    if (@deleted) {
        my $commit = 0;
        if ($nonstop) {
            $commit = 1;
        } else {
            my $a;
            do {
                print "\nAll done. Delete marked messages from the server? ([Y]es, [n]o): ";
                $a = <STDIN>;
            } until $a =~ /^[yn]?$/i;
            if ($a =~ /^y?$/i) {
                $commit = 1;
            } else {
                say "Aborted. No messages were deleted from the server.";
                $pop->quit;
                exit;
            }
        }
        if ($commit) {
            my $total_size = 0;
            foreach my $msgnum (@deleted) {
                $total_size += $messages{$msgnum};
                $pop->delete($msgnum);
            }
            say "@{[scalar @deleted]} deleted, total size was: @{[int($total_size/1024)]}K";
        }
    }
    say "@{[$nmails - @deleted]} left in the mailbox";
}
$pop->quit;
